@@ -1,4 +1,5 @@
 """Validation classes for various types of data."""
+
 from __future__ import annotations
 
 import re
@@ -7,9 +8,11 @@ from abc import ABC, abstractmethod
 from itertools import zip_longest
 from operator import attrgetter
 
-from marshmallow import types
 from marshmallow.exceptions import ValidationError
 
+if typing.TYPE_CHECKING:
+    from marshmallow import types
+
 _T = typing.TypeVar("_T")
 
 
@@ -21,15 +24,13 @@ class Validator(ABC):
         add a useful `__repr__` implementation for validators.
     """
 
-    error = None  # type: str | None
+    error: str | None = None
 
     def __repr__(self) -> str:
         args = self._repr_args()
         args = f"{args}, " if args else ""
 
-        return "<{self.__class__.__name__}({args}error={self.error!r})>".format(
-            self=self, args=args
-        )
+        return f"<{self.__class__.__name__}({args}error={self.error!r})>"
 
     def _repr_args(self) -> str:
         """A string representation of the args passed to this validator. Used by
@@ -38,8 +39,7 @@ class Validator(ABC):
         return ""
 
     @abstractmethod
-    def __call__(self, value: typing.Any) -> typing.Any:
-        ...
+    def __call__(self, value: typing.Any) -> typing.Any: ...
 
 
 class And(Validator):
@@ -49,42 +49,37 @@ class And(Validator):
 
         from marshmallow import validate, ValidationError
 
+
         def is_even(value):
             if value % 2 != 0:
                 raise ValidationError("Not an even value.")
 
+
         validator = validate.And(validate.Range(min=0), is_even)
         validator(-1)
         # ValidationError: ['Must be greater than or equal to 0.', 'Not an even value.']
 
     :param validators: Validators to combine.
-    :param error: Error message to use when a validator returns ``False``.
     """
 
-    default_error_message = "Invalid value."
-
-    def __init__(self, *validators: types.Validator, error: str | None = None):
+    def __init__(self, *validators: types.Validator):
         self.validators = tuple(validators)
-        self.error = error or self.default_error_message  # type: str
 
     def _repr_args(self) -> str:
         return f"validators={self.validators!r}"
 
     def __call__(self, value: typing.Any) -> typing.Any:
-        errors = []
-        kwargs = {}
+        errors: list[str | dict] = []
+        kwargs: dict[str, typing.Any] = {}
         for validator in self.validators:
             try:
-                r = validator(value)
-                if not isinstance(validator, Validator) and r is False:
-                    raise ValidationError(self.error)
+                validator(value)
             except ValidationError as err:
                 kwargs.update(err.kwargs)
                 if isinstance(err.messages, dict):
                     errors.append(err.messages)
                 else:
-                    # FIXME : Get rid of cast
-                    errors.extend(typing.cast(list, err.messages))
+                    errors.extend(err.messages)
         if errors:
             raise ValidationError(errors, **kwargs)
         return value
@@ -107,7 +102,7 @@ class URL(Validator):
             self._memoized = {}
 
         def _regex_generator(
-            self, relative: bool, absolute: bool, require_tld: bool
+            self, *, relative: bool, absolute: bool, require_tld: bool
         ) -> typing.Pattern:
             hostname_variants = [
                 # a normal domain name, expressed in [A-Z0-9] chars with hyphens allowed only in the middle
@@ -133,9 +128,9 @@ class URL(Validator):
                     # this is validated separately against allowed schemes, so in the regex
                     # we simply want to capture its existence
                     r"(?:[a-z0-9\.\-\+]*)://",
-                    # basic_auth, for URLs encoding a username:password
+                    # userinfo, for URLs encoding authentication
                     # e.g. 'ftp://foo:bar@ftp.example.org/'
-                    r"(?:[^:@]+?(:[^:@]*?)?@|)",
+                    r"(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?",
                     # netloc, the hostname/domain part of the URL plus the optional port
                     r"(?:",
                     "|".join(hostname_variants),
@@ -161,12 +156,12 @@ class URL(Validator):
             return re.compile("".join(parts), re.IGNORECASE)
 
         def __call__(
-            self, relative: bool, absolute: bool, require_tld: bool
+            self, *, relative: bool, absolute: bool, require_tld: bool
         ) -> typing.Pattern:
             key = (relative, absolute, require_tld)
             if key not in self._memoized:
                 self._memoized[key] = self._regex_generator(
-                    relative, absolute, require_tld
+                    relative=relative, absolute=absolute, require_tld=require_tld
                 )
 
             return self._memoized[key]
@@ -191,8 +186,8 @@ class URL(Validator):
             )
         self.relative = relative
         self.absolute = absolute
-        self.error = error or self.default_message  # type: str
-        self.schemes = schemes or self.default_schemes
+        self.error: str = error or self.default_message
+        self.schemes = {s.lower() for s in schemes} if schemes else self.default_schemes
         self.require_tld = require_tld
 
     def _repr_args(self) -> str:
@@ -207,14 +202,24 @@ class URL(Validator):
             raise ValidationError(message)
 
         # Check first if the scheme is valid
+        scheme = None
         if "://" in value:
             scheme = value.split("://")[0].lower()
             if scheme not in self.schemes:
                 raise ValidationError(message)
 
-        regex = self._regex(self.relative, self.absolute, self.require_tld)
+        regex = self._regex(
+            relative=self.relative, absolute=self.absolute, require_tld=self.require_tld
+        )
 
-        if not regex.search(value):
+        # Hostname is optional for file URLS. If absent it means `localhost`.
+        # Fill it in for the validation if needed
+        if scheme == "file" and value.startswith("file:///"):
+            matched = regex.search(value.replace("file:///", "file://localhost/", 1))
+        else:
+            matched = regex.search(value)
+
+        if not matched:
             raise ValidationError(message)
 
         return value
@@ -237,7 +242,8 @@ class Email(Validator):
 
     DOMAIN_REGEX = re.compile(
         # domain
-        r"(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+" r"(?:[A-Z]{2,6}|[A-Z0-9-]{2,})\Z"
+        r"(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+"
+        r"(?:[A-Z]{2,6}|[A-Z0-9-]{2,})\Z"
         # literal form, ipv4 address (SMTP 4.1.3)
         r"|^\[(25[0-5]|2[0-4]\d|[0-1]?\d?\d)"
         r"(\.(25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}\]\Z",
@@ -249,7 +255,7 @@ class Email(Validator):
     default_message = "Not a valid email address."
 
     def __init__(self, *, error: str | None = None):
-        self.error = error or self.default_message  # type: str
+        self.error: str = error or self.default_message
 
     def _format_error(self, value: str) -> str:
         return self.error.format(input=value)
@@ -309,8 +315,8 @@ class Range(Validator):
 
     def __init__(
         self,
-        min=None,
-        max=None,
+        min=None,  # noqa: A002
+        max=None,  # noqa: A002
         *,
         min_inclusive: bool = True,
         max_inclusive: bool = True,
@@ -335,9 +341,7 @@ class Range(Validator):
         )
 
     def _repr_args(self) -> str:
-        return "min={!r}, max={!r}, min_inclusive={!r}, max_inclusive={!r}".format(
-            self.min, self.max, self.min_inclusive, self.max_inclusive
-        )
+        return f"min={self.min!r}, max={self.max!r}, min_inclusive={self.min_inclusive!r}, max_inclusive={self.max_inclusive!r}"
 
     def _format_error(self, value: _T, message: str) -> str:
         return (self.error or message).format(input=value, min=self.min, max=self.max)
@@ -358,6 +362,9 @@ class Range(Validator):
         return value
 
 
+_SizedT = typing.TypeVar("_SizedT", bound=typing.Sized)
+
+
 class Length(Validator):
     """Validator which succeeds if the value passed to it has a
     length between a minimum and maximum. Uses len(), so it
@@ -380,8 +387,8 @@ class Length(Validator):
 
     def __init__(
         self,
-        min: int | None = None,
-        max: int | None = None,
+        min: int | None = None,  # noqa: A002
+        max: int | None = None,  # noqa: A002
         *,
         equal: int | None = None,
         error: str | None = None,
@@ -400,12 +407,12 @@ class Length(Validator):
     def _repr_args(self) -> str:
         return f"min={self.min!r}, max={self.max!r}, equal={self.equal!r}"
 
-    def _format_error(self, value: typing.Sized, message: str) -> str:
+    def _format_error(self, value: _SizedT, message: str) -> str:
         return (self.error or message).format(
             input=value, min=self.min, max=self.max, equal=self.equal
         )
 
-    def __call__(self, value: typing.Sized) -> typing.Sized:
+    def __call__(self, value: _SizedT) -> _SizedT:
         length = len(value)
 
         if self.equal is not None:
@@ -437,7 +444,7 @@ class Equal(Validator):
 
     def __init__(self, comparable, *, error: str | None = None):
         self.comparable = comparable
-        self.error = error or self.default_message  # type: str
+        self.error: str = error or self.default_message
 
     def _repr_args(self) -> str:
         return f"comparable={self.comparable!r}"
@@ -478,7 +485,7 @@ class Regexp(Validator):
         self.regex = (
             re.compile(regex, flags) if isinstance(regex, (str, bytes)) else regex
         )
-        self.error = error or self.default_message  # type: str
+        self.error: str = error or self.default_message
 
     def _repr_args(self) -> str:
         return f"regex={self.regex!r}"
@@ -487,12 +494,10 @@ class Regexp(Validator):
         return self.error.format(input=value, regex=self.regex.pattern)
 
     @typing.overload
-    def __call__(self, value: str) -> str:
-        ...
+    def __call__(self, value: str) -> str: ...
 
     @typing.overload
-    def __call__(self, value: bytes) -> bytes:
-        ...
+    def __call__(self, value: bytes) -> bytes: ...
 
     def __call__(self, value):
         if self.regex.match(value) is None:
@@ -517,7 +522,7 @@ class Predicate(Validator):
 
     def __init__(self, method: str, *, error: str | None = None, **kwargs):
         self.method = method
-        self.error = error or self.default_message  # type: str
+        self.error: str = error or self.default_message
         self.kwargs = kwargs
 
     def _repr_args(self) -> str:
@@ -526,7 +531,7 @@ class Predicate(Validator):
     def _format_error(self, value: typing.Any) -> str:
         return self.error.format(input=value, method=self.method)
 
-    def __call__(self, value: typing.Any) -> typing.Any:
+    def __call__(self, value: _T) -> _T:
         method = getattr(value, self.method)
 
         if not method(**self.kwargs):
@@ -548,7 +553,7 @@ class NoneOf(Validator):
     def __init__(self, iterable: typing.Iterable, *, error: str | None = None):
         self.iterable = iterable
         self.values_text = ", ".join(str(each) for each in self.iterable)
-        self.error = error or self.default_message  # type: str
+        self.error: str = error or self.default_message
 
     def _repr_args(self) -> str:
         return f"iterable={self.iterable!r}"
@@ -588,7 +593,7 @@ class OneOf(Validator):
         self.choices_text = ", ".join(str(choice) for choice in self.choices)
         self.labels = labels if labels is not None else []
         self.labels_text = ", ".join(str(label) for label in self.labels)
-        self.error = error or self.default_message  # type: str
+        self.error: str = error or self.default_message
 
     def _repr_args(self) -> str:
         return f"choices={self.choices!r}, labels={self.labels!r}"
@@ -618,8 +623,7 @@ class OneOf(Validator):
         :param valuegetter: Can be a callable or a string. In the former case, it must
             be a one-argument callable which returns the value of a
             choice. In the latter case, the string specifies the name
-            of an attribute of the choice objects. Defaults to `str()`
-            or `str()`.
+            of an attribute of the choice objects. Defaults to `str()`.
         """
         valuegetter = valuegetter if callable(valuegetter) else attrgetter(valuegetter)
         pairs = zip_longest(self.choices, self.labels, fillvalue="")
@@ -632,9 +636,9 @@ class ContainsOnly(OneOf):
     in the sequence is also in the sequence passed as ``choices``. Empty input
     is considered valid.
 
-    :param iterable choices: Same as :class:`OneOf`.
-    :param iterable labels: Same as :class:`OneOf`.
-    :param str error: Same as :class:`OneOf`.
+    :param choices: Same as :class:`OneOf`.
+    :param labels: Same as :class:`OneOf`.
+    :param error: Same as :class:`OneOf`.
 
     .. versionchanged:: 3.0.0b2
         Duplicate values are considered valid.
@@ -662,8 +666,8 @@ class ContainsNoneOf(NoneOf):
     in the sequence is a member of the sequence passed as ``iterable``. Empty input
     is considered valid.
 
-    :param iterable iterable: Same as :class:`NoneOf`.
-    :param str error: Same as :class:`NoneOf`.
+    :param iterable: Same as :class:`NoneOf`.
+    :param error: Same as :class:`NoneOf`.
 
     .. versionadded:: 3.6.0
     """
