@@ -14,6 +14,7 @@ from marshmallow import (
     RAISE,
     Schema,
     fields,
+    missing,
     validate,
 )
 from marshmallow.exceptions import ValidationError
@@ -1005,6 +1006,13 @@ class TestFieldDeserialization:
         field = fields.Function(lambda x: None, deserialize=lambda val: val.upper())
         assert field.deserialize("foo") == "FOO"
 
+    def test_function_field_deserialization_missing_with_length_validator(self):
+        field = fields.Function(
+            deserialize=lambda value: value.get("some-key", missing),
+            validate=validate.Length(min=0),
+        )
+        assert field.deserialize({}) is missing
+
     def test_function_field_passed_deserialize_only_is_load_only(self):
         field = fields.Function(deserialize=lambda val: val.upper())
         assert field.load_only is True
@@ -1306,7 +1314,7 @@ class TestFieldDeserialization:
         field = fields.List(fields.DateTime())
         result = field.deserialize(dstrings)
         assert all(isinstance(each, dt.datetime) for each in result)
-        for actual, expected in zip(result, dtimes):
+        for actual, expected in zip(result, dtimes, strict=True):
             assert_date_equal(actual, expected)
 
     def test_list_field_deserialize_invalid_item(self):
@@ -1347,7 +1355,9 @@ class TestFieldDeserialization:
 
         assert isinstance(result, tuple)
         assert len(result) == 2
-        for val, type_, true_val in zip(result, (dt.datetime, int), (dtime, 42)):
+        for val, type_, true_val in zip(
+            result, (dt.datetime, int), (dtime, 42), strict=True
+        ):
             assert isinstance(val, type_)
             assert val == true_val
 
